using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;

// ThermalKatanaEditor v1.0
//
// Allows you to edit some parameters of ThermalKatana particles

namespace ThermalKatanaEditorPlugin
{
    public class ThermalKatanaEditor : MVRScript
    {
		public JSONStorableColor ThermalKatanaMainColor;
		public JSONStorableFloat ThermalKatanaType;
		public JSONStorableBool ThermalKatanaEnableTrails;
		public JSONStorableBool ThermalKatanaEnableGlow;
		public JSONStorableBool ThermalKatanaEnableLightning;
		public JSONStorableBool ThermalKatanaEnableTendrils;
		
		public Transform ObjectRoot;
		public Transform HiltsRoot;
		public Transform currentHilt;
		private string objectNameSearchString = "ThermalKatana";

		private bool seInitialized = false;
		
		private Vector3 curPos;
		private Vector3 lastPos;
		private Quaternion curRot;
		private Quaternion lastRot;
		
		// Security
		private int initLoops = 0;
		private int maxInitLoops = 3600;
		
		public override void Init()
        {			 
            try
            {
				if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("Please add ThermalKatanasEditor on an Custom Unity Asset with a compatible particle system.");
                    return;
                }

				HSVColor hsvc = HSVColorPicker.RGBToHSV(0.75f, 0.19f, 0f);
				ThermalKatanaMainColor = new JSONStorableColor("ThermalKatana color", hsvc, MainColorCallback);                
				RegisterColor(ThermalKatanaMainColor);
				CreateColorPicker(ThermalKatanaMainColor);

				ThermalKatanaEnableTrails = new JSONStorableBool("Enable Trails When Moving", false, EnableVFXCallback);
				RegisterBool(ThermalKatanaEnableTrails);
				CreateToggle(ThermalKatanaEnableTrails, false);
							
				ThermalKatanaEnableGlow = new JSONStorableBool("Enable Trails", false, EnableVFXCallback);
				RegisterBool(ThermalKatanaEnableGlow);
				CreateToggle(ThermalKatanaEnableGlow, false);

				ThermalKatanaEnableLightning = new JSONStorableBool("Enable Sparks", true, EnableVFXCallback);
				RegisterBool(ThermalKatanaEnableLightning);
				CreateToggle(ThermalKatanaEnableLightning, false);
				
				ThermalKatanaEnableTendrils = new JSONStorableBool("Enable Tendrils", false, EnableVFXCallback);
				RegisterBool(ThermalKatanaEnableTendrils);
				CreateToggle(ThermalKatanaEnableTendrils, false);		
							
				JSONStorableString helpText = new JSONStorableString("Help",
					"<color=#000><size=35><b>Help</b></size></color>\n\n" + 
					"<color=#333>" +
					"<b>ThermalKatanasEditor</b> is a tiny plugin that allows you to edit some parameters of the ThermalKatana inside this package\n\n" +
					"Just test the parameters on the left, and the ThermalKatana will update in realtime.\n\n" +
					"</color>"
				);
				UIDynamic helpTextfield = CreateTextField(helpText, true);
				helpTextfield.height = 550.0f;


			}
            catch(Exception e)
            {
                SuperController.LogError("ThermalKatanasEditor - Exception caught: " + e);
            }
		}

		void Start()
        {

		}	

		void Update()
        {
			// If we haven't made over 1min of init tests, we aren't initialized and the particle system (CUA) is still compatible, try to initialize
			if( initLoops < maxInitLoops && seInitialized == false ) {
				initLoops++;
				InitObjects();
			}
			
			if( initLoops == maxInitLoops ) {
				logError("ThermalKatanasEditor : This CUA particle could not be initialized properly after 3600 tries. Please select a compatible particle and reload the script.");
				initLoops++; // To avoid spamming the log
			}

		}

		void FixedUpdate() {
			try {
				// enable the trails when the object is moving
				if (ThermalKatanaEnableTrails.val)
				{
					ThermalKatanaEnableGlow.val = is_moving();
				}
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		private bool is_moving(){
			curPos = ObjectRoot.position;
			curRot = ObjectRoot.rotation;
			if(curPos != lastPos || curRot != lastRot) {
				lastPos = curPos;
				lastRot = curRot;
				return true;
			}
			else{
				lastPos = curPos;
				lastRot = curRot;
				return false;
			}
		}

		private bool InitObjects() {
			if( seInitialized == true ) return true;
			
			ObjectRoot = findObjectRoot( containingAtom.reParentObject );
			if( ObjectRoot == null ) {
				return false;
			}
			
			HiltsRoot = getObjectRoot( ObjectRoot, "hilts" );
			if( HiltsRoot != null ) {
				seInitialized = true;
				
				TypeCallback( 1 );
				
				MainColorCallback( ThermalKatanaMainColor );
				
								
				logDebug("ThermalKatanasEditor : ThermalKatana editor initialized properly");
							
				return true;
			} else {
				logError("ThermalKatanasEditor : your CUA is not a ThermalKatana.");
				initLoops = maxInitLoops; // We don't have a particle system, we consider the initialization failed
				return false;
			}
		}
		
		// Finds the object containing a portion of a name
		private Transform findObjectRoot( Transform parent ) {
			foreach( Transform child in parent )
			{
				if( child.name.Contains(objectNameSearchString) ) {
					return child;
				} else {
					Transform childSearch = findObjectRoot(child);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}
		
		// Finds the object with an exact name
		private Transform getObjectRoot( Transform parent, string objectName ) {
			foreach( Transform child in parent )
			{
				if( child.name == objectName ) {
					return child;
				} else {
					Transform childSearch = getObjectRoot(child, objectName);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}
		
		protected void MainColorCallback(JSONStorableColor color) {
			if( currentHilt ) {
				initBladeGlow();
				initThermalKatana();
			}
		}
		
		protected void TypeCallback(float type) {
			if( HiltsRoot ) {
				disableAllHilts();
				currentHilt = enableHilt(type);
				initBladeGlow();
				initThermalKatana();
			}
		}
		
		protected void EnableVFXCallback(bool enabled) {
			if( currentHilt ) {
				initBladeGlow();
				initVisualEffects();
			}
		}
		
		protected void disableAllHilts() {
			foreach( Transform child in HiltsRoot )
			{
				child.gameObject.SetActive(false);
			}
		}
		
		protected Transform enableHilt(float hiltValue) {
			foreach( Transform child in HiltsRoot )
			{
				if( child.name == "hilt_0" + hiltValue ) {
					child.gameObject.SetActive(true);
					
					return child;
				}
			}
			
			return null;
		}
		
		protected void initThermalKatana() {
			initBladeColor();
			initBladeGlow();
			initVisualEffects();
		}
		
		protected void initBladeColor() {
			Transform blade = getObjectRoot( currentHilt, "blade" );
			if( blade ) {
				Renderer bladeRenderer = blade.GetComponent<Renderer>();
				// bladeRenderer.material.SetColor("_EmissionColor",new Color(ThermalKatanaMainColor.colorPicker.currentColor.r, ThermalKatanaMainColor.colorPicker.currentColor.g, ThermalKatanaMainColor.colorPicker.currentColor.b, 1.0f) );
			}
		}
		
		protected void initBladeGlow() {
			Transform bladeGlow = getObjectRoot( currentHilt, "blade_glow" );
			if( bladeGlow ) {
				bladeGlow.gameObject.SetActive( ThermalKatanaEnableGlow.val );
				ParticleSystem prtComponent = bladeGlow.GetComponent<ParticleSystem>();
				if( prtComponent ) {
					Color newColor = new Color(ThermalKatanaMainColor.colorPicker.currentColor.r, ThermalKatanaMainColor.colorPicker.currentColor.g, ThermalKatanaMainColor.colorPicker.currentColor.b, 1.0f);
					var prtMain = prtComponent.main;
					prtMain.startColor = newColor;
					prtComponent.Clear();
					prtComponent.Play();
				}
			}
		}
		
		protected void initVisualEffects() {
			Transform bladeLightning = getObjectRoot( currentHilt, "blade_lightning" );
			if( bladeLightning ) {
				bladeLightning.gameObject.SetActive( ThermalKatanaEnableLightning.val );
				ParticleSystem prtComponentLightning = bladeLightning.GetComponent<ParticleSystem>();
				if( prtComponentLightning ) {
					Color newColor = new Color(ThermalKatanaMainColor.colorPicker.currentColor.r, ThermalKatanaMainColor.colorPicker.currentColor.g, ThermalKatanaMainColor.colorPicker.currentColor.b, 1.0f);
					var prtMain = prtComponentLightning.main;
					prtMain.startColor = newColor;
					prtComponentLightning.Clear();
					prtComponentLightning.Play();
				}
			}
			
			Transform bladeTendrils = getObjectRoot( currentHilt, "blade_tendrils" );
			if( bladeTendrils ) {
				bladeTendrils.gameObject.SetActive( ThermalKatanaEnableTendrils.val );
				ParticleSystem prtComponentTendrils = bladeTendrils.GetComponent<ParticleSystem>();
				if( prtComponentTendrils ) {
					Color newColor = new Color(ThermalKatanaMainColor.colorPicker.currentColor.r, ThermalKatanaMainColor.colorPicker.currentColor.g, ThermalKatanaMainColor.colorPicker.currentColor.b, 1.0f);
					var prtMain = prtComponentTendrils.main;
					prtMain.startColor = newColor;
					prtComponentTendrils.Clear();
					prtComponentTendrils.Play();
				}
			}
		}
		
		// **************************
		// Local Tools
		// **************************
		private void logDebug( string debugText ) {
			SuperController.LogMessage( debugText );
		}
		
		private void logError( string debugText ) {
			SuperController.LogError( debugText );
		}
	}
}